import { Dialog, showDialog, ToolbarButton } from "@jupyterlab/apputils";
import { PanelLayout, Widget } from '@lumino/widgets';
import { PageConfig, URLExt } from "@jupyterlab/coreutils";
import { ServerConnection } from "@jupyterlab/services";
import { LabIcon } from '@jupyterlab/ui-components';
import * as React from 'react';
import Utils from './utils';
import showBrowseFileDialog from './BrowseFileDialog';
import schedulerSvgstr from '../style/icon/scheduler.svg';
const schedulerIcon = new LabIcon({
    name: 'ui-components:scheduler',
    svgstr: schedulerSvgstr
});
/**
 * Submit notebook button extension
 *  - Attach button to notebook toolbar and launch a dialog requesting
 *  information about the remote location to where submit the notebook
 *  for execution
 */
export class SubmitNotebookButtonExtension {
    constructor(app, browserFactory) {
        this.showWidget = () => {
            let envVars = Utils.getEnvVars(this.panel.content.model.toString());
            // let notebookName = this.context.path.replace(/(.*).ipynb/, '$1');
            let notebookName = this.context.path;
            showDialog({
                title: 'Submit notebook',
                body: new SubmitNotebook(envVars, notebookName, this.browserFactory),
                buttons: [Dialog.cancelButton(), Dialog.okButton()]
            }).then(result => {
                console.log();
                if (result.value == null) {
                    // When Cancel is clicked on the dialog, just return
                    return;
                }
                // prepare notebook submission details
                let notebookTask = result.value;
                notebookTask.kernelspec = "python3";
                notebookTask.notebook_name = this.context.path;
                notebookTask.notebook_full_path = PageConfig.getOption('serverRoot') + '/' + this.context.path;
                notebookTask.notebook = this.panel.content.model.toJSON();
                notebookTask.root_path = PageConfig.getOption('serverRoot') + '/';
                // use ServerConnection utility to make calls to Jupyter Based services
                // which in this case is the scheduler extension installed by this package
                let settings = ServerConnection.makeSettings();
                let url = URLExt.join(settings.baseUrl, 'jhub_scheduler', 'scheduler');
                let requestBody = JSON.stringify(notebookTask);
                ServerConnection.makeRequest(url, { method: 'POST', body: requestBody }, settings)
                    .then(response => {
                    if (response.status !== 200) {
                        return response.json().then(data => {
                            showDialog({
                                title: "Error submitting Notebook !",
                                body: data.message,
                                buttons: [Dialog.okButton()]
                            });
                        });
                    }
                    return response.json();
                })
                    .then(data => {
                    if (data) {
                        let dialogTitle = 'Job submission to the TTSS';
                        if (data['status'] == 'ok') {
                            let dialogLink = React.createElement('a', { href: data['url'].replace('/&', '&'), target: "_blank" }, 'Console & Job Status');
                            let dialogBody = React.createElement('p', null, 'Check details on submitted jobs at ', dialogLink);
                            dialogTitle = dialogTitle + ' succeeded !';
                            showDialog({
                                title: dialogTitle,
                                body: dialogBody,
                                buttons: [Dialog.okButton()]
                            });
                        }
                        else {
                            dialogTitle = dialogTitle + ' failed !';
                            showDialog({
                                title: dialogTitle,
                                body: data['message'],
                                buttons: [Dialog.okButton()]
                            });
                        }
                    }
                });
            });
        };
        this.app = app;
        this.browserFactory = browserFactory;
    }
    createNew(panel, context) {
        this.panel = panel;
        this.context = context;
        // Create the toolbar button
        let submitNotebookButton = new ToolbarButton({
            // className: 'fa fa-send',
            // label: 'Submit Notebook ...',
            icon: schedulerIcon,
            onClick: this.showWidget,
            tooltip: 'Submit Notebook ...'
        });
        // Add the toolbar button to the notebook
        panel.toolbar.insertItem(10, 'submitNotebook', submitNotebookButton);
        // The ToolbarButton class implements `IDisposable`, so the
        // button *is* the extension for the purposes of this method.
        return submitNotebookButton;
    }
}
/**
 * Submit notebook dialog extension
 * - Request information about the remote location to where submit the
 * notebook for execution
 */
export class SubmitNotebook extends Widget {
    constructor(envVars, notebookName, browserFactory) {
        super();
        this._envVars = envVars;
        this._notebookName = notebookName.replace(/(.*).ipynb/, '$1');
        this.browserFactory = browserFactory;
        console.log('notebookName: ' + notebookName);
        console.log('this._notebookName: ' + this._notebookName);
        this._htmlDialogElement = this.renderHtml(this._notebookName);
        this._htmlDialogElement.getElementsByTagName('button').item(0).onclick = () => {
            console.log('dependencies_list_button click event triggered!');
            console.log('startPath: ' + PageConfig.getOption('serverRoot'));
            console.log('rootPath: ' + this.browserFactory.defaultBrowser.model.path);
            showBrowseFileDialog(this.browserFactory.defaultBrowser.model.manager, {
                multiselect: true,
                includeDir: true,
                startPath: PageConfig.getOption('serverRoot'),
                rootPath: this.browserFactory.defaultBrowser.model.path || '/Users/happyelements/GitlabProjects/jupyterlab_apod',
            }).then((result) => {
                if (result.button.accept && result.value.length) {
                    // (<HTMLTextAreaElement>document.getElementById('dependencies_list')).value = result.value[0].path;
                    // console.log('xxxx: ' + result.value[0].path);
                    const dependenciesSet = new Set(document.getElementById('dependencies_list').value.split('\n'));
                    result.value.forEach((val) => {
                        dependenciesSet.add(val.path);
                    });
                    document.getElementById('dependencies_list').value = [...dependenciesSet].join('\n');
                }
            });
        };
        let layout = (this.layout = new PanelLayout());
        layout.addWidget(new Widget({ node: this._htmlDialogElement.firstElementChild }));
    }
    /**
     * Render the dialog widget used to gather configuration information
     * required to submit/run the notebook remotely
     */
    renderHtml(notebook_name) {
        var tr = '<tr style="padding: 1px;">';
        var td = '<td style="padding: 1px;">';
        var td_colspan2 = '<td style="padding: 1px;" colspan=2>';
        // var td_colspan3 = '<td style="padding: 1px;" colspan=3>';
        //var td_colspan4 = '<td style="padding: 1px;" colspan=4>';
        var content = ''
            + '<table id="table-submit-dialog"><tbody>'
            + tr
            + td
            + '<label for="pipeline_name">Pipeline name:</label>'
            + '<br/>'
            + '<input type="text" id="pipeline_name" name="pipeline_name" placeholder="' + notebook_name + '" value="' + notebook_name + '"/>'
            + '<br/>'
            + '</td>'
            + td
            + '<label for="vm_label">VM label:</label>'
            + '<br/>'
            + '<select id="vm_label"><option value="he_cpu_1" selected>he_cpu_1</option><option value="he_cpu_4_32_0">he_cpu_4_32_0</option><option value="he_cpu_32_128_0">he_cpu_32_128_0</option></select>'
            + '</td>'
            + '</tr>'
            + tr
            + td
            + '<label for="schedule_express">Notebook job schedule express:</label>'
            + '</td>'
            + td
            + '<p><a target="_blank" href="https://www.quartz-scheduler.org/api/2.1.7/org/quartz/CronExpression.html">Cron expression reference</a></p>'
            + '</td>'
            + '</tr>'
            + tr
            + td
            + '<br/>'
            + '<input type="text" id="schedule_express" name="schedule_express" placeholder="0 30 10 * * ?" value="0 30 10 * * ?" size="20"/>'
            + '<br/>'
            + '</td>'
            + td
            + '<br/>'
            + '<input type="checkbox" id="execute_now" name="execute_now" size="20"/> Execute once on submit<br/>'
            + '</td>'
            + '</tr>'
            + tr
            + td
            + '<br/>'
            + '<input type="checkbox" id="dependency_include" name="dependency_include" size="20" checked="checked"/> Include dependencies<br/>'
            + '</td>'
            + td
            + '<br/>'
            + '<button id="dependencies_list_button" name="dependencies_list_button">Browse dependency dir</button>'
            + '</td>'
            + '</tr>'
            + tr
            + td_colspan2
            + '<label for="schedule_express">Dependencies list:</label>'
            + '<br/>'
            + '<textarea id="dependencies_list" name="dependencies_list" rows="5" cols="35"></textarea>'
            + '</td>'
            + '</tr>'
            + this.getEnvHtml()
            + '</tbody></table>';
        let htmlContent = document.createElement('div');
        htmlContent.innerHTML = content;
        return htmlContent;
    }
    getEnvHtml() {
        let tr = '<tr style="padding: 1px;">';
        let td = '<td style="padding: 1px;">';
        let td_colspan4 = '<td style="padding: 1px;" colspan=4>';
        // let td_colspan2 = '<td style="padding: 1px;" colspan=2>';
        let subtitle = '<div style="font-size: var(--jp-ui-font-size3)">Environmental Variables</div>';
        if (this._envVars.length > 0) {
            let html = '' + tr + td_colspan4 + subtitle + '</td>' + '</tr>';
            for (let i = 0; i < this._envVars.length; i++) {
                if (i % 4 === 0) {
                    html = html + tr;
                }
                html = html + td
                    + `<label for="envVar${i}">${this._envVars[i]}:</label>`
                    + '<br/>'
                    + `<input type="text" id="envVar${i}" class="envVar" name="envVar${i}" placeholder="" value="" size="20"/>`
                    + '</td>';
                if (i % 4 === 3) {
                    html = html + '</tr>';
                }
            }
            return html;
        }
        else {
            return '';
        }
    }
    getValue() {
        let dependency_list = '';
        console.log('dependency_include: ' + document.getElementById('dependency_include').checked);
        if (document.getElementById('dependency_include').checked) {
            dependency_list = document.getElementById('dependencies_list').value.split('\n').join(',');
            // dependency_list = (<HTMLInputElement>document.getElementById('dependencies_list')).value
            console.log('dependency_list: ' + dependency_list);
        }
        let envVars = {};
        let envElements = document.getElementsByClassName('envVar');
        for (let i = 0; i < envElements.length; i++) {
            let index = parseInt(envElements[i].id.match(/\d+/)[0], 10);
            envVars[this._envVars[index]] = envElements[i].value;
        }
        let returnData = {
            // platform: (<HTMLSelectElement> document.getElementById('platform')).value,
            // endpoint: (<HTMLInputElement> document.getElementById('endpoint')).value,
            // user: (<HTMLInputElement>document.getElementById('user')).value,
            // userinfo: (<HTMLInputElement>document.getElementById('userinfo')).value,
            // framework: (<HTMLSelectElement> document.getElementById('framework')).value,
            // cpus: Number((<HTMLInputElement>document.getElementById('cpus')).value),
            // gpus: Number((<HTMLInputElement>document.getElementById('gpus')).value),
            // memory: (<HTMLInputElement>document.getElementById('memory')).value,
            dependencies: dependency_list,
            // cos_endpoint: (<HTMLInputElement>document.getElementById('cos_endpoint')).value,
            // cos_user: (<HTMLInputElement>document.getElementById('cos_user')).value,
            // cos_password: (<HTMLInputElement>document.getElementById('cos_password')).value,
            // cos_bucket_in: (<HTMLInputElement>document.getElementById('cos_bucket_in')).value,
            // cos_bucket_out: (<HTMLInputElement>document.getElementById('cos_bucket_out')).value,
            vm_label: document.getElementById('vm_label').value,
            pipeline_name: document.getElementById('pipeline_name').value,
            schedule_express: document.getElementById('schedule_express').value,
            execute_now: Boolean(document.getElementById('execute_now').checked),
            env: envVars,
        };
        return returnData;
    }
}
